"use client"

import React, { useEffect, useState, FormEvent } from 'react';
import { AddressElement, Elements } from '@stripe/react-stripe-js';
import { Appearance, StripeAddressElementOptions, StripeElementsOptions, loadStripe } from '@stripe/stripe-js';
import {
  PaymentElement,
  useStripe,
  useElements,
} from '@stripe/react-stripe-js';
import { RadioGroup } from '@headlessui/react'

const stripe = loadStripe(process.env.NEXT_PUBLIC_STRIPE_PUBLISHABLE_KEY || '');

const addressOptions: StripeAddressElementOptions = {
  mode: 'billing',
  allowedCountries: ['US'],
  fields: {
    phone: 'always',
  },
  validation: {
    phone: {
      required: 'always'
    }
  },
  defaultValues: {
    address: {
      state: 'WA',
      country: 'US',
    },
  }
};

const membershipLevels = [
  { id: 1, title: 'Individual', description: 'One vote', price: 20 },
  { id: 2, title: 'Household', description: 'Two votes', price: 30 },
];

const additionalDonationLevels = [10, 50, 100, -1];

function classNames(...classes: string[]) {
  return classes.filter(Boolean).join(' ');
}

function Spinner(): React.JSX.Element {
  return (<svg xmlns="http://www.w3.org/2000/svg" width="24" height="24" viewBox="0 0 24 24">
    <path fill="currentColor" d="M10.72,19.9a8,8,0,0,1-6.5-9.79A7.77,7.77,0,0,1,10.4,4.16a8,8,0,0,1,9.49,6.52A1.54,1.54,0,0,0,21.38,12h.13a1.37,1.37,0,0,0,1.38-1.54,11,11,0,1,0-12.7,12.39A1.54,1.54,0,0,0,12,21.34h0A1.47,1.47,0,0,0,10.72,19.9Z">
      <animateTransform attributeName="transform" dur="1s" repeatCount="indefinite" type="rotate" values="0 12 12;360 12 12" />
    </path>
  </svg>);
}

function CheckoutForm({
  paymentIntentID
}: {
  paymentIntentID: string
}): React.JSX.Element {
  const [selectedMembershipLevel, setSelectedMembershipLevel] = useState(membershipLevels[0]);
  const [selectedAdditionalDonation, setSelectedAdditionalDonation] = useState<number | null>(null);
  const [customAmount, setCustomAmount] = useState('');
  const [email, setEmail] = useState('');
  const [offsetFees, setOffsetFees] = useState(true);
  const [totalAmount, setTotalAmount] = useState(300);
  const [message, setMessage] = useState<string>('');
  const [isLoading, setIsLoading] = useState(false);
  const stripe = useStripe();
  const elements = useElements();

  useEffect(() => {
    if (!stripe) {
      return;
    }

    //Grab the client secret from url params
    const clientSecret = new URLSearchParams(window.location.search).get(
      'payment_intent_client_secret'
    );

    if (!clientSecret) {
      return;
    }

    stripe.retrievePaymentIntent(clientSecret).then(({ paymentIntent }) => {
      switch (paymentIntent?.status) {
        case 'succeeded':
          setMessage('Payment succeeded!');
          break;
        case 'processing':
          setMessage('Your payment is processing.');
          break;
        case 'requires_payment_method':
          setMessage('Your payment was not successful, please try again.');
          break;
        default:
          setMessage('Something went wrong.');
          break;
      }
    });
  }, [stripe]);

  useEffect(() => {
    let subtotal = selectedMembershipLevel.price

    if (selectedAdditionalDonation && selectedAdditionalDonation !== -1) {
      subtotal += selectedAdditionalDonation
    } else if (customAmount !== '') {
      try {
        subtotal += parseFloat(customAmount)
      } catch {
        console.error('')
      }
    }

    if (offsetFees) {
      subtotal = Math.ceil(subtotal * 1.03)
    }

    setTotalAmount(subtotal);

    fetch('api/stripe_intent', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify({
        amount: subtotal * 100,
        payment_intent_id: paymentIntentID,
        metadata: {
          'type': selectedMembershipLevel.title,
        },
      }),
    });
  }, [paymentIntentID, selectedMembershipLevel, selectedAdditionalDonation, customAmount, offsetFees, email])

  const handleSubmit = async (e: FormEvent) => {
    e.preventDefault();

    if (!stripe || !elements) {
      // Stripe.js has not yet loaded.
      return;
    }

    setIsLoading(true);

    const { error } = await stripe.confirmPayment({
      elements,
      confirmParams: {
        return_url: window.location.origin + '/thank-you',
        receipt_email: email,
        payment_method_data: {
          billing_details: {
            // Other details are filled automatically by address form.
            email,
          },
        },
      },
    });

    if (error.type === 'card_error' || error.type === 'validation_error') {
      setMessage(error.message || '');
    } else {
      setMessage('An unexpected error occured.');
    }

    setIsLoading(false);
  };

  return (
    <>
      <form id="payment-form" onSubmit={handleSubmit} className="m-auto space-y-4">

        {/* Membership Type */}
        <RadioGroup value={selectedMembershipLevel} onChange={setSelectedMembershipLevel} className="space-y-3">
          <RadioGroup.Label className="text-base font-semibold leading-6 text-gray-900">
            Select a membership type
          </RadioGroup.Label>

          <div className="grid grid-cols-1 gap-y-6 sm:grid-cols-2 sm:gap-x-4">
            {membershipLevels.map((membership) => (
              <RadioGroup.Option
                key={membership.id}
                value={membership}
                className={({ active }) =>
                  classNames(
                    active ? 'border-indigo-600 ring-2 ring-indigo-600' : 'border-gray-300',
                    'relative flex cursor-pointer rounded-lg border bg-white p-4 shadow-sm focus:outline-none'
                  )
                }
              >
                {({ checked, active }) => (
                  <>
                    <span className="flex flex-1 items-center justify-between ">
                      <span className="flex flex-col">
                        <RadioGroup.Label as="span" className="block text-sm font-medium text-gray-900">
                          {membership.title}
                        </RadioGroup.Label>
                        <RadioGroup.Description as="span" className="mt-1 flex items-center text-sm text-gray-500">
                          {membership.description}
                        </RadioGroup.Description>
                      </span>
                      <RadioGroup.Description as="span" className="ml-8 text-sm font-medium">
                        <span className="text-gray-900">${membership.price}</span>
                        <span className="text-gray-500">/yr</span>
                      </RadioGroup.Description>
                    </span>
                    <span
                      className={classNames(
                        active ? 'border' : 'border-2',
                        checked ? 'border-indigo-600' : 'border-transparent',
                        'pointer-events-none absolute -inset-px rounded-lg'
                      )}
                      aria-hidden="true"
                    />
                  </>
                )}
              </RadioGroup.Option>
            ))}
          </div>
        </RadioGroup>

        {/* Additional donation */}
        <RadioGroup value={selectedAdditionalDonation} onChange={setSelectedAdditionalDonation} className="space-y-3">
          <RadioGroup.Label className="text-base font-semibold leading-6 text-gray-900">
            Additional donation
          </RadioGroup.Label>
          <div className="grid grid-cols-3 gap-3 sm:grid-cols-5">
            {additionalDonationLevels.map((option) => (
              <RadioGroup.Option
                key={option}
                value={option}
                className={({ active, checked }) =>
                  classNames(
                    'cursor-pointer focus:outline-none',
                    option === -1 ? 'col-span-2' : '',
                    checked
                      ? 'ring-2 ring-indigo-600 ring-offset-2'
                      : 'ring-1 ring-inset ring-gray-300 bg-white text-gray-900 hover:bg-gray-50',
                    'flex items-center justify-center rounded-md py-3 px-3 text-sm font-semibold sm:flex-1'
                  )
                }
              >

                {option === -1
                  ? (
                    <RadioGroup.Label as="span" className="flex flex-col items-center">
                      <div className="relative rounded-md shadow-sm">
                        <div className="pointer-events-none absolute inset-y-0 left-0 flex items-center pl-3">
                          <span className="text-gray-500 sm:text-sm">$</span>
                        </div>
                        <input
                          type="number"
                          name="custom-amount"
                          id="custom-amount"
                          value={customAmount}
                          min="0"
                          step="1"
                          onChange={(e) => setCustomAmount(e.target.value)}
                          className="block w-full rounded-md border-0 py-1.5 pl-7 pr-12 text-gray-900 ring-1 ring-inset ring-gray-300 placeholder:text-gray-400 focus:ring-1 focus:ring-inset focus:ring-gray-500 sm:text-sm sm:leading-6"
                          placeholder="Custom"
                          aria-describedby="price-currency"
                        />
                        <div className="pointer-events-none absolute inset-y-0 right-0 flex items-center pr-3">
                          <span className="text-gray-500 sm:text-sm" id="price-currency">
                            /yr
                          </span>
                        </div>
                      </div>
                    </RadioGroup.Label>
                  )
                  : (
                    <RadioGroup.Label as="span">
                      <span>${option}</span>
                      <span className="text-gray-500">/yr</span>
                    </RadioGroup.Label>
                  )}
              </RadioGroup.Option>
            ))}
          </div>
        </RadioGroup>

        <div className="space-y-3">
          <h2 className="text-base font-semibold leading-6 text-gray-900">
            About you
          </h2>

          <div className="rounded-md mb-3 px-3 pb-1.5 pt-2.5 shadow-sm ring-1 ring-inset ring-gray-300 focus-within:ring-2 focus-within:ring-indigo-600">
            <label htmlFor="name" className="block text-xs font-medium text-gray-900">
              Email
            </label>
            <input
              type="email"
              name="email"
              id="email"
              value={email}
              onChange={(e) => setEmail(e.target.value)}
              className="block w-full border-0 p-0 text-gray-900 placeholder:text-gray-400 focus:ring-0 sm:text-sm sm:leading-6"
              placeholder="you@example.com"
            />
          </div>
          <AddressElement options={addressOptions} />
        </div>

        <div className="space-y-3">
          <h2 className="text-base font-semibold leading-6 text-gray-900">
            Payment
          </h2>
          <div className="mt-1 text-sm text-gray-500">
            If you would like to pay by cash or check, please instead
            <a className="underline mx-1" href="/WSCC-Membership-Form.pdf">fill out a paper form</a>
            and mail to the address on the form.
          </div>
          <PaymentElement id="payment-element" />
        </div>

        {/* TODO: Automatically renew toggle? */}

        <div className="relative flex gap-x-3">
          <div className="flex h-6 items-center">
            <input
              id="offsetFees"
              name="offsetFees"
              type="checkbox"
              checked={offsetFees}
              onChange={(e) => setOffsetFees(e.target.checked)}
              className="h-4 w-4 rounded border-gray-300 text-indigo-600 focus:ring-indigo-600"
            />
          </div>
          <div className="text-sm leading-6">
            <label htmlFor="offsetFees" className="font-medium text-gray-900">
              Help offset credit card fees
            </label>
          </div>
        </div>

        <button
          className="mt-6 w-full rounded-md flex justify-center border border-transparent bg-indigo-600 px-4 py-2 text-sm font-medium text-white shadow-sm hover:bg-indigo-700 focus:outline-none focus:ring-2 focus:ring-indigo-500 focus:ring-offset-2"
          disabled={isLoading || !stripe || !elements}
          id="submit"
        >
          {isLoading ? (
            <Spinner />
          ) : (
            'Pay $' + totalAmount
          )}
        </button>

        {/* Show any error or success messages */}
        {message && <div className="text-red-500" id="payment-message">{message}</div>}
      </form>
    </>
  );
}

export function ClubPayment() {
  const [clientSecret, setClientSecret] = useState('');
  const [paymentIntent, setPaymentIntent] = useState('');
  useEffect(() => {
    // Create PaymentIntent as soon as the page loads using our local API
    fetch('api/stripe_intent', {
      method: 'POST',
      headers: { 'Content-Type': 'application/json' },
      body: JSON.stringify({
        amount: 2000,
        payment_intent_id: '',
      }),
    })
      .then((res) => res.json())
      .then((data) => {
        setClientSecret(data.client_secret);
        setPaymentIntent(data.id);
      });
  }, []);

  const appearance: Appearance = {
    theme: 'stripe',
    labels: 'floating',
  };

  const options: StripeElementsOptions = {
    clientSecret,
    appearance
  }
  return <>
    {clientSecret && (
      <Elements options={options}
        stripe={stripe}>
        <CheckoutForm paymentIntentID={paymentIntent} />
      </Elements>
    )}
  </>
}


