import Image, { type ImageProps } from 'next/image'
import clsx from 'clsx'

import { Button } from '@/components/Button'
import { Card } from '@/components/Card'
import { Container } from '@/components/Container'
import { CalendarDaysIcon, EnvelopeIcon } from '@heroicons/react/24/solid'
import exteriorFrontImage from '@/images/photos/exterior-front.png'
import stageImage from '@/images/photos/stage.jpg'
import exteriorSoutheastImage from '@/images/photos/exterior-southeast.jpg'
import interorEmptyImage from '@/images/photos/interior-empty.jpg'
import kitchenImage from '@/images/photos/kitchen.jpg'
import { type BlogPostWithSlug, getAllBlogPosts } from '@/lib/articles'
import { formatDate } from '@/lib/formatDate'

function Article({ article }: { article: BlogPostWithSlug }) {
  return (
    <Card as="article">
      <Card.Title href={`/blog/${article.slug}`}>
        {article.title}
      </Card.Title>
      <Card.Eyebrow as="time" dateTime={article.date} decorate>
        {formatDate(article.date)}
      </Card.Eyebrow>
      <Card.Description>{article.description}</Card.Description>
      <Card.Cta>Read article</Card.Cta>
    </Card>
  )
}

function Newsletter() {
  return (
    <form
      action="/thank-you"
      className="rounded-2xl border border-zinc-100 p-6 dark:border-zinc-700/40"
    >
      <h2 className="flex text-sm font-semibold text-zinc-900 dark:text-zinc-100">
        <EnvelopeIcon className="h-6 w-6 flex-none fill-zinc-100 stroke-zinc-400 dark:fill-zinc-100/10 dark:stroke-zinc-500" />
        <span className="ml-3">Stay up to date</span>
      </h2>
      <p className="mt-2 text-sm text-zinc-600 dark:text-zinc-400">
        Get notified about upcoming events and stay in touch with the community.
      </p>
      <div className="mt-6 flex">
        <input
          type="email"
          placeholder="Email address"
          aria-label="Email address"
          required
          className="min-w-0 flex-auto appearance-none rounded-md border border-zinc-900/10 bg-white px-3 py-[calc(theme(spacing.2)-1px)] shadow-md shadow-zinc-800/5 placeholder:text-zinc-400 focus:border-teal-500 focus:outline-none focus:ring-4 focus:ring-teal-500/10 dark:border-zinc-700 dark:bg-zinc-700/[0.15] dark:text-zinc-200 dark:placeholder:text-zinc-500 dark:focus:border-teal-400 dark:focus:ring-teal-400/10 sm:text-sm"
        />
        <Button type="submit" className="ml-4 flex-none">
          Join
        </Button>
      </div>
    </form>
  )
}

interface Meeting {
  title: string
  date: string
  startTime: string
  endTime?: string
  notes?: string
}

function MeetingListItem({ meeting }: { meeting: Meeting }) {
  return (
    <li className="flex gap-4">
      <dl className="flex flex-auto flex-wrap gap-x-2">
        <dt className="sr-only">Title</dt>
        <dd className="w-full flex-none text-sm font-medium text-zinc-900 dark:text-zinc-100">
          {meeting.title}
        </dd>
        <dt className="sr-only">Date</dt>
        <dd className="text-xs text-zinc-500 dark:text-zinc-400">
          {meeting.date}
        </dd>
        <dt className="sr-only">Time</dt>
        {meeting.endTime
          ? <dd
            className="ml-auto text-xs text-zinc-400 dark:text-zinc-500"
            aria-label={`${meeting.startTime} until ${meeting.endTime}`}
          >
            <time dateTime={meeting.date + ' ' + meeting.startTime}>{meeting.startTime}</time>{' '}
            <span aria-hidden="true">—</span>{' '}
            <time dateTime={meeting.date + ' ' + meeting.endTime}>{meeting.endTime}</time>{' '}
          </dd>
          : <dd
            className="ml-auto text-xs text-zinc-400 dark:text-zinc-500"
            aria-label={`${meeting.startTime}`}
          >
            <time dateTime={meeting.date + ' ' + meeting.startTime}>{meeting.startTime}</time>{' '}
          </dd>
        }
      </dl>
    </li>
  )
}

function Events() {
  let events: Array<Meeting> = [
    {
      title: 'January Potluck',
      date: '2024-01-20',
      startTime: '6:00pm',
    },
    {
      title: 'February Potluck',
      date: '2024-02-17',
      startTime: '6:00pm',
      // endTime: '8:00pm',
      // notes: 'Bring your own chair.'
    },
  ]

  return (
    <div className="rounded-2xl border border-zinc-100 p-6 dark:border-zinc-700/40">
      <h2 className="flex text-sm font-semibold text-zinc-900 dark:text-zinc-100">
        <CalendarDaysIcon className="h-6 w-6 flex-none fill-zinc-100 stroke-zinc-400 dark:fill-zinc-100/10 dark:stroke-zinc-500" />
        <span className="ml-3">Upcoming Events</span>
      </h2>
      <ol className="mt-6 space-y-4">
        {events.map((meeting, idx) => (
          <MeetingListItem key={idx} meeting={meeting} />
        ))}
      </ol>
      {/* 
      <Button href="#" variant="secondary" className="group mt-6 w-full">
        Full Calendar
      </Button>
      */}
    </div>
  )
}

function Photos() {
  let rotations = ['rotate-2', '-rotate-2', 'rotate-2', 'rotate-2', '-rotate-2']

  return (
    <div className="mt-16 sm:mt-20">
      <div className="-my-4 flex justify-center gap-5 overflow-hidden py-4 sm:gap-8">
        {[exteriorSoutheastImage, stageImage, exteriorFrontImage, interorEmptyImage, kitchenImage].map((image, imageIndex) => (
          <div
            key={image.src}
            className={clsx(
              'relative aspect-[9/10] w-44 flex-none overflow-hidden rounded-xl bg-zinc-100 dark:bg-zinc-800 sm:w-72 sm:rounded-2xl',
              rotations[imageIndex % rotations.length],
            )}
          >
            <Image
              src={image}
              alt=""
              sizes="(min-width: 640px) 18rem, 11rem"
              className="absolute inset-0 h-full w-full object-cover"
            />
          </div>
        ))}
      </div>
    </div>
  )
}

export default async function Home() {
  let articles = (await getAllBlogPosts()).slice(0, 4)

  return (
    <>
      <Container className="mt-9">
        <div className="max-w-2xl">
          <h1 className="text-4xl font-bold tracking-tight text-zinc-800 dark:text-zinc-100 sm:text-5xl">
            West Sound Community Hall
          </h1>
          <p className="mt-6 text-base text-zinc-600 dark:text-zinc-400">
            The West Sound Community Hall is located in the hamlet of West Sound
            on Orcas Island, about 10 minutes from the ferry landing and
            Eastsound. It has served as a public assembly hall since it was
            built by volunteers in 1902.
          </p>
          <p className="mt-6 text-base text-zinc-600 dark:text-zinc-400">
            Facing West Sound, the Hall is at the heart of the West Sound community.
          </p>
        </div>
      </Container>
      <Photos />
      <Container className="mt-24 md:mt-28">
        <div className="mx-auto grid max-w-xl grid-cols-1 gap-y-20 lg:max-w-none lg:grid-cols-2">
          <div className="flex flex-col gap-16">
            {articles.map((article) => (
              <Article key={article.slug} article={article} />
            ))}
          </div>
          <div className="space-y-10 lg:pl-16 xl:pl-24">
            <Newsletter />
            <Events />
          </div>
        </div>
      </Container>
    </>
  )
}
