'use client'

import { Fragment, useEffect, useRef, useState } from 'react'
import Link from 'next/link'
import { usePathname } from 'next/navigation'
import { useTheme } from 'next-themes'
import { Popover, Transition } from '@headlessui/react'
import clsx from 'clsx'
import { XMarkIcon, ChevronDownIcon } from '@heroicons/react/24/solid'

import { Container } from '@/components/Container'

function SunIcon(props: React.ComponentPropsWithoutRef<'svg'>) {
  return (
    <svg
      viewBox="0 0 24 24"
      strokeWidth="1.5"
      strokeLinecap="round"
      strokeLinejoin="round"
      aria-hidden="true"
      {...props}
    >
      <path d="M8 12.25A4.25 4.25 0 0 1 12.25 8v0a4.25 4.25 0 0 1 4.25 4.25v0a4.25 4.25 0 0 1-4.25 4.25v0A4.25 4.25 0 0 1 8 12.25v0Z" />
      <path
        d="M12.25 3v1.5M21.5 12.25H20M18.791 18.791l-1.06-1.06M18.791 5.709l-1.06 1.06M12.25 20v1.5M4.5 12.25H3M6.77 6.77 5.709 5.709M6.77 17.73l-1.061 1.061"
        fill="none"
      />
    </svg>
  )
}

function MoonIcon(props: React.ComponentPropsWithoutRef<'svg'>) {
  return (
    <svg viewBox="0 0 24 24" aria-hidden="true" {...props}>
      <path
        d="M17.25 16.22a6.937 6.937 0 0 1-9.47-9.47 7.451 7.451 0 1 0 9.47 9.47ZM12.75 7C17 7 17 2.75 17 2.75S17 7 21.25 7C17 7 17 11.25 17 11.25S17 7 12.75 7Z"
        strokeWidth="1.5"
        strokeLinecap="round"
        strokeLinejoin="round"
      />
    </svg>
  )
}

function MobileNavItem({
  href,
  children,
}: {
  href: string
  children: React.ReactNode
}) {
  return (
    <li>
      <Popover.Button as={Link} href={href} className="block py-2">
        {children}
      </Popover.Button>
    </li>
  )
}

function MobileNavigation(
  props: React.ComponentPropsWithoutRef<typeof Popover>,
) {
  return (
    <Popover {...props}>
      <Popover.Button className="group flex items-center rounded-full bg-white/90 px-4 py-2 text-sm font-medium text-zinc-800 shadow-lg shadow-zinc-800/5 ring-1 ring-zinc-900/5 backdrop-blur dark:bg-zinc-800/90 dark:text-zinc-200 dark:ring-white/10 dark:hover:ring-white/20">
        Menu
        <ChevronDownIcon className="ml-3 h-auto w-2 stroke-zinc-500 group-hover:stroke-zinc-700 dark:group-hover:stroke-zinc-400" />
      </Popover.Button>
      <Transition.Root>
        <Transition.Child
          as={Fragment}
          enter="duration-150 ease-out"
          enterFrom="opacity-0"
          enterTo="opacity-100"
          leave="duration-150 ease-in"
          leaveFrom="opacity-100"
          leaveTo="opacity-0"
        >
          <Popover.Overlay className="fixed inset-0 z-50 bg-zinc-800/40 backdrop-blur-sm dark:bg-black/80" />
        </Transition.Child>
        <Transition.Child
          as={Fragment}
          enter="duration-150 ease-out"
          enterFrom="opacity-0 scale-95"
          enterTo="opacity-100 scale-100"
          leave="duration-150 ease-in"
          leaveFrom="opacity-100 scale-100"
          leaveTo="opacity-0 scale-95"
        >
          <Popover.Panel
            focus
            className="fixed inset-x-4 top-8 z-50 origin-top rounded-3xl bg-white p-8 ring-1 ring-zinc-900/5 dark:bg-zinc-900 dark:ring-zinc-800"
          >
            <div className="flex flex-row-reverse items-center justify-between">
              <Popover.Button aria-label="Close menu" className="-m-1 p-1">
                <XMarkIcon className="h-6 w-6 text-zinc-500 dark:text-zinc-400" />
              </Popover.Button>
              <h2 className="text-sm font-medium text-zinc-600 dark:text-zinc-400">
                Navigation
              </h2>
            </div>
            <nav className="mt-6">
              <ul className="-my-2 divide-y divide-zinc-100 text-base text-zinc-800 dark:divide-zinc-100/5 dark:text-zinc-300">
                <MobileNavItem href="/hall-history">Hall History</MobileNavItem>
                <MobileNavItem href="/blog">Blog</MobileNavItem>
                <MobileNavItem href="/rental">Rental</MobileNavItem>
                <MobileNavItem href="/club">Club</MobileNavItem>
              </ul>
            </nav>
          </Popover.Panel>
        </Transition.Child>
      </Transition.Root>
    </Popover>
  )
}

function NavItem({
  href,
  children,
}: {
  href: string
  children: React.ReactNode
}) {
  let isActive = usePathname() === href

  return (
    <li>
      <Link
        href={href}
        className={clsx(
          'relative block px-3 py-2 transition',
          isActive
            ? 'text-teal-500 dark:text-teal-400'
            : 'hover:text-teal-500 dark:hover:text-teal-400',
        )}
      >
        {children}
        {isActive && (
          <span className="absolute inset-x-1 -bottom-px h-px bg-gradient-to-r from-teal-500/0 via-teal-500/40 to-teal-500/0 dark:from-teal-400/0 dark:via-teal-400/40 dark:to-teal-400/0" />
        )}
      </Link>
    </li>
  )
}

function DesktopNavigation(props: React.ComponentPropsWithoutRef<'nav'>) {
  return (
    <nav {...props}>
      <ul className="flex rounded-full bg-white/90 px-3 text-sm font-medium text-zinc-800 shadow-lg shadow-zinc-800/5 ring-1 ring-zinc-900/5 backdrop-blur dark:bg-zinc-800/90 dark:text-zinc-200 dark:ring-white/10">
        <NavItem href="/hall-history">History</NavItem>
        <NavItem href="/blog">Blog</NavItem>
        <NavItem href="/rental">Rental</NavItem>
        <NavItem href="/club">Club</NavItem>
      </ul>
    </nav>
  )
}

function ThemeToggle() {
  let { resolvedTheme, setTheme } = useTheme()
  let otherTheme = resolvedTheme === 'dark' ? 'light' : 'dark'
  let [mounted, setMounted] = useState(false)

  useEffect(() => {
    setMounted(true)
  }, [])

  return (
    <button
      type="button"
      aria-label={mounted ? `Switch to ${otherTheme} theme` : 'Toggle theme'}
      className="group rounded-full bg-white/90 px-3 py-2 shadow-lg shadow-zinc-800/5 ring-1 ring-zinc-900/5 backdrop-blur transition dark:bg-zinc-800/90 dark:ring-white/10 dark:hover:ring-white/20"
      onClick={() => setTheme(otherTheme)}
    >
      <SunIcon className="h-6 w-6 fill-zinc-100 stroke-zinc-500 transition group-hover:fill-zinc-200 group-hover:stroke-zinc-700 dark:hidden [@media(prefers-color-scheme:dark)]:fill-teal-50 [@media(prefers-color-scheme:dark)]:stroke-teal-500 [@media(prefers-color-scheme:dark)]:group-hover:fill-teal-50 [@media(prefers-color-scheme:dark)]:group-hover:stroke-teal-600" />
      <MoonIcon className="hidden h-6 w-6 fill-zinc-700 stroke-zinc-500 transition dark:block [@media(prefers-color-scheme:dark)]:group-hover:stroke-zinc-400 [@media_not_(prefers-color-scheme:dark)]:fill-teal-400/10 [@media_not_(prefers-color-scheme:dark)]:stroke-teal-500" />
    </button>
  )
}

function clamp(number: number, a: number, b: number) {
  let min = Math.min(a, b)
  let max = Math.max(a, b)
  return Math.min(Math.max(number, min), max)
}

function AvatarContainer({
  className,
  ...props
}: React.ComponentPropsWithoutRef<'div'>) {
  return (
    <div
      className={clsx(
        className,
        'h-10 w-10 rounded-full bg-white/90 p-0.5 shadow-lg shadow-zinc-800/5 ring-1 ring-zinc-900/5 backdrop-blur dark:bg-zinc-800/90 dark:ring-white/10',
      )}
      {...props}
    />
  )
}

function Avatar({
  large = false,
  className,
  ...props
}: Omit<React.ComponentPropsWithoutRef<typeof Link>, 'href'> & {
  large?: boolean
}) {
  return (
    <Link
      href="/"
      aria-label="Home"
      className={clsx(className, 'pointer-events-auto')}
      {...props}
    >
      <svg
        version="1.1"
        viewBox="0 0 62.009 50.971"
        xmlns="http://www.w3.org/2000/svg"
        className={clsx(
          'rounded-full bg-zinc-100 text-zinc-800 object-cover dark:bg-zinc-400 dark:text-teal-900',
          large ? 'h-16 w-16' : 'h-9 w-9',
        )}
        fill="currentColor"
      >
        <g transform="translate(-48.899 -51.537)">
          <path
            d="m85.112 64.259c-4.0662 3.8864-8.2522 8.1726-12.308 12.228v-6.3624h-3.1027v0.01757h-0.03514v19.422c0.81208 0.83451 1.7 1.6947 2.4505 2.4365v10.031h-9.202v0.47593h47.993v-0.47593h-5.3439v-12.416h0.09353v-0.81856h-0.09353v-1.4764l0.16536-0.25632-1.0459-0.95085 1.0211-0.024805 0.00827-0.81029c-6.8599-6.9555-13.745-14.02-20.601-21.019zm-1.2578 2.1704 0.045992 0.046509-11.096 12.01v-1.0061l11.05-11.05zm0.57102 0.58291 12.908 13.178v21.842h-24.529c2.4e-5 -7.4708-7e-6 -14.938 0-22.442l11.621-12.578zm-13.912 3.7822h1.604v6.38l-1.604 1.604v-7.984zm1.604 7.3722v1.0625l-1.604 1.7363v-1.1948l1.604-1.604zm0 2.1678v10.557l-1.604-1.5865v-7.2347l1.604-1.7358zm25.824 0.47594c0.3526 0.35062 6.9736 6.8534 7.2497 7.091v0.89555h-7.2497v-7.9866zm-4.733 0.003101-17.57 0.033073-0.52917 2.7864 1.7942-0.0031 0.024804 11.595 16.024-0.016536v-11.606l1.4883-0.002584-1.232-2.7864zm-16.979 0.66146h16.345l0.74672 1.4304h-17.386l0.29404-1.4304zm10.968 2.3812h0.83509v6.565h4.192v4.1258h-14.626v-4.1258h0.49609 4.192 0.71934 4.192v-6.565zm-8.2553 0.28629 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.248 0 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.2485 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm2.4148 0 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.248 0 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.2485 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm2.5306 0 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.248 0 0.50023 0.50023-0.50023 0.49971-0.49971-0.49971 0.49971-0.50023zm1.2485 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm-11.81 0.62425 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm3.6633 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm3.7791 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.49971-0.50023-0.49971 0.50023-0.50023zm-11.811 0.62425 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.4148 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.5306 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm-11.81 0.62374 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm3.6633 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm3.7791 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm-11.811 0.62425 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.4148 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.5306 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm-11.81 0.62374 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm3.6633 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm3.7791 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm1.248 0 0.49971 0.50023-0.49971 0.50023-0.50023-0.50023 0.50023-0.50023zm-11.811 0.62425 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.4148 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.5306 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm-11.81 0.62425 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm1.248 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm3.6633 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm1.248 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm3.7791 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm1.248 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm-11.811 0.62374 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.4148 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm2.5306 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.248 0 0.50023 0.49971-0.50023 0.50023-0.49971-0.50023 0.49971-0.49971zm1.2485 0 0.49971 0.49971-0.49971 0.50023-0.50023-0.50023 0.50023-0.49971zm6.5645 0.48059h1.2511v4.5134h-1.2511v-4.5134zm1.9761 0h0.88884v4.5134h-0.88884v-4.5134zm1.6139 0h1.017v4.5134h-1.017v-4.5134zm1.742 0h1.9177v4.5134h-1.9177v-4.5134zm-5.332 5.1449h6.6802l-1.1849 1.2511h-5.4953v-1.2511zm7.0885 0h0.16123v1.2511h-1.1436l1.0868-1.1472-0.10439-0.10387zm-7.0885 1.8826h4.8969l-1.5952 1.6836h-3.3016v-1.6836zm5.5077 0h1.742v1.6836h-3.3373l1.5952-1.6836zm-5.5077 2.3151h2.7032l-1.1519 1.2159h-1.5513v-1.2159zm3.3145 0h3.9352v1.2159h-5.0875l1.1524-1.2159zm-3.3145 1.8474h0.95291l-0.95291 1.0061v-1.0061zm1.5637 0h5.686v1.2263h-6.8476l1.1617-1.2263z"
            strokeWidth=".165"
          />
          <g strokeWidth=".26458">
            <path
              d="m80.397 79.597h-2.2214v-0.41892h0.85438v-2.7506h-0.85438v-0.37483q0.17363 0 0.37207-0.02756 0.19844-0.03032 0.30041-0.08544 0.12678-0.0689 0.19844-0.17363 0.07441-0.10749 0.08544-0.28663h0.42719v3.6987h0.83785zm3.8172-2.2986q0 0.53743-0.12402 0.97565-0.12127 0.43822-0.36931 0.7469-0.2508 0.31419-0.62838 0.48507-0.37758 0.17088-0.88746 0.17088-0.14332 0-0.2701-0.01654-0.12678-0.01378-0.226-0.04685v-0.52641h0.02756q0.07993 0.04134 0.226 0.07993 0.14607 0.03583 0.32522 0.03583 0.60909 0 0.9536-0.3638 0.34726-0.36656 0.40239-1.0087-0.25631 0.15434-0.48231 0.22049-0.226 0.06615-0.49334 0.06615-0.25356 0-0.46026-0.04961-0.20395-0.04961-0.41066-0.19293-0.24254-0.16812-0.36656-0.42719-0.12127-0.25907-0.12127-0.62012 0-0.62838 0.41341-1.0225 0.41341-0.39412 1.0087-0.39412 0.29766 0 0.55122 0.09371 0.25356 0.09095 0.44373 0.27561 0.23427 0.22875 0.36104 0.5898 0.12678 0.35829 0.12678 0.9288zm-0.55673-0.11851q0-0.42719-0.08819-0.68626t-0.24253-0.40239q-0.12954-0.12402-0.27836-0.17639-0.14883-0.05512-0.32246-0.05512-0.39688 0-0.62838 0.24805-0.22875 0.24805-0.22875 0.69729 0 0.26183 0.07441 0.42719 0.07441 0.16536 0.2508 0.28939 0.12402 0.08544 0.27285 0.11576 0.14883 0.02756 0.33073 0.02756 0.21497 0 0.43546-0.05788t0.41341-0.16812q0.0028-0.05788 0.0055-0.113 0.0055-0.05788 0.0055-0.14607zm4.1451 0.3638q0 1.1052-0.34727 1.6233-0.34451 0.51539-1.0721 0.51539-0.73863 0-1.0804-0.52365-0.339-0.52365-0.339-1.6095 0-1.0942 0.34451-1.6151 0.34451-0.52365 1.0749-0.52365 0.73863 0 1.0776 0.53192 0.34175 0.52917 0.34175 1.6013zm-0.72485 1.2513q0.09646-0.22324 0.12954-0.52365 0.03583-0.30317 0.03583-0.7276 0-0.41892-0.03583-0.7276-0.03307-0.30868-0.13229-0.52365-0.09646-0.21222-0.26458-0.3197-0.16536-0.10749-0.42719-0.10749-0.25907 0-0.42995 0.10749-0.16812 0.10749-0.26734 0.32522-0.09371 0.20395-0.12954 0.53192-0.03307 0.32797-0.03307 0.71934 0 0.42995 0.03032 0.71934t0.12954 0.51814q0.09095 0.21497 0.25632 0.32797 0.16812 0.113 0.44373 0.113 0.25907 0 0.42995-0.10749 0.17088-0.10749 0.26458-0.32522zm4.3133-1.2513q0 1.1052-0.34726 1.6233-0.34451 0.51539-1.0721 0.51539-0.73863 0-1.0804-0.52365-0.339-0.52365-0.339-1.6095 0-1.0942 0.34451-1.6151 0.34451-0.52365 1.0749-0.52365 0.73863 0 1.0776 0.53192 0.34175 0.52917 0.34175 1.6013zm-0.72485 1.2513q0.09646-0.22324 0.12954-0.52365 0.03583-0.30317 0.03583-0.7276 0-0.41892-0.03583-0.7276-0.03307-0.30868-0.13229-0.52365-0.09646-0.21222-0.26458-0.3197-0.16536-0.10749-0.42719-0.10749-0.25907 0-0.42995 0.10749-0.16812 0.10749-0.26734 0.32522-0.09371 0.20395-0.12954 0.53192-0.03307 0.32797-0.03307 0.71934 0 0.42995 0.03032 0.71934t0.12954 0.51814q0.09095 0.21497 0.25632 0.32797 0.16812 0.113 0.44373 0.113 0.25907 0 0.42995-0.10749 0.17088-0.10749 0.26458-0.32522z"
              aria-label="1900"
            />
            <path
              d="m66.477 92.817c0.0887-1.2733 0.32752-2.9023 0.53071-3.6199 0.20319-0.71764 0.36944-1.5808 0.36944-1.9182 0-0.3374 0.1228-0.61344 0.27289-0.61344 0.54517 0 0.60438 0.92473 0.1682 2.6269-0.24504 0.95629-0.44453 2.3102-0.44331 3.0086 0.0019 1.108-0.56539 2.8311-0.93216 2.8311-0.06987 0-0.05447-1.0418 0.03423-2.3151zm-0.85596-2.5135c0.15648-1.161 0.69544-8.8559 0.78144-11.157 0.03123-0.8354 0.1791-1.3445 0.41316-1.4224 0.30312-0.10086 0.33312 0.43028 0.17807 3.1532-0.10255 1.8009-0.17972 4.0484-0.17149 4.9942 0.02167 2.4886-0.51981 5.5562-0.98073 5.5562-0.28504 0-0.33658-0.26288-0.22045-1.1245zm-7.5264-0.38126c-0.03623-2.091-0.84883-7.7665-1.1913-8.3206-0.06612-0.10699-0.35976-1.5415-0.65252-3.1877-0.7668-4.3118-0.82784-4.4384-3.3084-6.8596-2.0236-1.9751-2.6513-2.8805-1.9973-2.8805 0.13023 0 0.9339 0.71358 1.7859 1.5857 1.7328 1.7737 1.946 1.8753 1.946 0.92782 0-0.36381 0.15145-0.66146 0.33656-0.66146 0.32603 0 0.77768 1.7031 1.6839 6.35 0.80992 4.1529 0.87506 4.244 0.75665 1.0583-0.1107-2.9782-0.44713-4.7446-1.4614-7.6729-0.27721-0.80036-0.48961-1.6933-0.47199-1.9844 0.05121-0.84622-0.7367-3.1482-1.4813-4.3276-0.72224-1.1441-0.85064-1.6255-0.43357-1.6255 0.73674 0 2.6585 3.9897 2.6585 5.5192 0 0.36432 0.29524 1.504 0.65609 2.5326 0.81637 2.327 1.4072 5.7099 1.3121 7.5133-0.08406 1.595 0.03595 1.7022 0.544 0.4863 0.37874-0.90645 3.7086-4.6742 4.131-4.6742 0.63515 0 0.07472 1.0146-1.4391 2.6054-0.92127 0.96812-1.8325 2.1563-2.0249 2.6404-0.23849 0.60002-0.48715 0.84425-0.78127 0.76734-0.46071-0.12048-0.34968 1.8261 0.25712 4.5078 0.33446 1.4782 0.32834 4.7485-0.0093 4.9571-0.14552 0.0899-0.26458 0.50666-0.26458 0.92604 0 0.41939-0.11906 0.83611-0.26458 0.92604-0.16137 0.0997-0.27318-0.33282-0.28663-1.1089zm9.7053-12.583c-0.10535-4.2565-0.30379-8.3939-0.44098-9.1943-0.13719-0.80037-0.35428-2.2886-0.48241-3.3073-0.27188-2.1614-0.48472-2.7781-0.95873-2.7781-0.27864 0-2.2449-3.1365-2.2449-3.581 0-0.0678 0.21949-0.12319 0.48776-0.12319 0.34355 0 0.73166 0.52493 1.3127 1.7754l0.8249 1.7754 0.0764-2.6139c0.1146-3.9208 0.67052-3.718 0.83083 0.30302 0.15841 3.9733 0.38332 5.3746 0.86262 5.3746 0.20397 0 1.4119-0.40592 2.6842-0.90204 3.1575-1.2312 3.883-1.341 8.2003-1.2407 2.306 0.0535 1.0865 0.55528-1.3496 0.55528-2.6163 0-2.9345 0.0718-6.5862 1.4856-1.2723 0.49261-2.5593 0.89566-2.8599 0.89566-0.30949 0-0.4731 0.11883-0.37722 0.27398 0.71518 1.1572 1.2855 19.041 0.60724 19.041-0.33756 0-0.42355-1.1337-0.58704-7.7391zm20.479-10.938c0-0.0862 0.22668-0.44488 0.50374-0.7971 1.0618-1.3499 0.29267-3.6696-1.7564-5.2972-0.69348-0.55084-1.5089-1.2701-1.812-1.5983-0.30313-0.32824-0.87643-0.72048-1.274-0.87163-0.8232-0.31298-5.0165-0.37876-4.8691-0.0764 0.05318 0.10914 0.26228 0.52909 0.46466 0.93322 0.2575 0.51418 0.28439 0.81837 0.08957 1.0132s-0.5698-0.20127-1.249-1.3193c-3.4598-5.6955-9.6455-7.6721-14.487-4.6292-1.9924 1.2522-4.1791 5.644-3.5889 7.2078 0.29954 0.79355 0.13402 1.623-0.32388 1.623-0.15009 0-0.27289-0.30032-0.27289-0.66737 0-0.77791-1.6737-2.7066-3.3489-3.8592-0.93888-0.64597-1.3725-0.76504-2.7842-0.76461l-1.6721 5.1e-4 -0.84782 1.2563c-0.4663 0.69095-0.96937 1.5837-1.1179 1.9839-0.14856 0.40018-0.45165 0.7276-0.67354 0.7276-0.54456 0-0.5275-0.0799 0.43417-2.0333 1.1569-2.3499 1.7079-2.7292 3.9658-2.7292 1.999 0 3.0615 0.48488 4.5726 2.0867 1.0228 1.0842 1.1183 1.0542 1.5785-0.49521 2.1542-7.2539 10.973-8.8212 17.03-3.0266l1.7766 1.6997h2.4323c2.8318 0 3.4984 0.20498 4.9884 1.5339 0.60415 0.53882 1.5349 1.3128 2.0683 1.72 1.5511 1.184 2.348 4.4138 1.4862 6.024-0.24492 0.45763-1.3136 0.7465-1.3136 0.35507z"
            />
          </g>
        </g>
      </svg>

    </Link>
  )
}

export function Header() {
  let isHomePage = usePathname() === '/'

  let headerRef = useRef<React.ElementRef<'div'>>(null)
  let avatarRef = useRef<React.ElementRef<'div'>>(null)
  let isInitial = useRef(true)

  useEffect(() => {
    let downDelay = avatarRef.current?.offsetTop ?? 0
    let upDelay = 64

    function setProperty(property: string, value: string) {
      document.documentElement.style.setProperty(property, value)
    }

    function removeProperty(property: string) {
      document.documentElement.style.removeProperty(property)
    }

    function updateHeaderStyles() {
      if (!headerRef.current) {
        return
      }

      let { top, height } = headerRef.current.getBoundingClientRect()
      let scrollY = clamp(
        window.scrollY,
        0,
        document.body.scrollHeight - window.innerHeight,
      )

      if (isInitial.current) {
        setProperty('--header-position', 'sticky')
      }

      setProperty('--content-offset', `${downDelay}px`)

      if (isInitial.current || scrollY < downDelay) {
        setProperty('--header-height', `${downDelay + height}px`)
        setProperty('--header-mb', `${-downDelay}px`)
      } else if (top + height < -upDelay) {
        let offset = Math.max(height, scrollY - upDelay)
        setProperty('--header-height', `${offset}px`)
        setProperty('--header-mb', `${height - offset}px`)
      } else if (top === 0) {
        setProperty('--header-height', `${scrollY + height}px`)
        setProperty('--header-mb', `${-scrollY}px`)
      }

      if (top === 0 && scrollY > 0 && scrollY >= downDelay) {
        setProperty('--header-inner-position', 'fixed')
        removeProperty('--header-top')
        removeProperty('--avatar-top')
      } else {
        removeProperty('--header-inner-position')
        setProperty('--header-top', '0px')
        setProperty('--avatar-top', '0px')
      }
    }

    function updateAvatarStyles() {
      if (!isHomePage) {
        return
      }

      let fromScale = 1
      let toScale = 36 / 64
      let fromX = 0
      let toX = 2 / 16

      let scrollY = downDelay - window.scrollY

      let scale = (scrollY * (fromScale - toScale)) / downDelay + toScale
      scale = clamp(scale, fromScale, toScale)

      let x = (scrollY * (fromX - toX)) / downDelay + toX
      x = clamp(x, fromX, toX)

      setProperty(
        '--avatar-image-transform',
        `translate3d(${x}rem, 0, 0) scale(${scale})`,
      )

      let borderScale = 1 / (toScale / scale)
      let borderX = (-toX + x) * borderScale
      let borderTransform = `translate3d(${borderX}rem, 0, 0) scale(${borderScale})`

      setProperty('--avatar-border-transform', borderTransform)
      setProperty('--avatar-border-opacity', scale === toScale ? '1' : '0')
    }

    function updateStyles() {
      updateHeaderStyles()
      updateAvatarStyles()
      isInitial.current = false
    }

    updateStyles()
    window.addEventListener('scroll', updateStyles, { passive: true })
    window.addEventListener('resize', updateStyles)

    return () => {
      window.removeEventListener('scroll', updateStyles)
      window.removeEventListener('resize', updateStyles)
    }
  }, [isHomePage])

  return (
    <>
      <header
        className="pointer-events-none relative z-50 flex flex-none flex-col"
        style={{
          height: 'var(--header-height)',
          marginBottom: 'var(--header-mb)',
        }}
      >
        {isHomePage && (
          <>
            <div
              ref={avatarRef}
              className="order-last mt-[calc(theme(spacing.16)-theme(spacing.3))]"
            />
            <Container
              className="top-0 order-last -mb-3 pt-3"
              style={{
                position:
                  'var(--header-position)' as React.CSSProperties['position'],
              }}
            >
              <div
                className="top-[var(--avatar-top,theme(spacing.3))] w-full"
                style={{
                  position:
                    'var(--header-inner-position)' as React.CSSProperties['position'],
                }}
              >
                <div className="relative">
                  <AvatarContainer
                    className="absolute left-0 top-3 origin-left transition-opacity"
                    style={{
                      opacity: 'var(--avatar-border-opacity, 0)',
                      transform: 'var(--avatar-border-transform)',
                    }}
                  />
                  <Avatar
                    large
                    className="block h-16 w-16 origin-left"
                    style={{ transform: 'var(--avatar-image-transform)' }}
                  />
                </div>
              </div>
            </Container>
          </>
        )}
        <div
          ref={headerRef}
          className="top-0 z-10 h-16 pt-6"
          style={{
            position:
              'var(--header-position)' as React.CSSProperties['position'],
          }}
        >
          <Container
            className="top-[var(--header-top,theme(spacing.6))] w-full"
            style={{
              position:
                'var(--header-inner-position)' as React.CSSProperties['position'],
            }}
          >
            <div className="relative flex gap-4">
              <div className="flex flex-1">
                {!isHomePage && (
                  <AvatarContainer>
                    <Avatar />
                  </AvatarContainer>
                )}
              </div>
              <div className="flex flex-1 justify-end md:justify-center">
                <MobileNavigation className="pointer-events-auto md:hidden" />
                <DesktopNavigation className="pointer-events-auto hidden md:block" />
              </div>
              <div className="flex justify-end md:flex-1">
                <div className="pointer-events-auto">
                  <ThemeToggle />
                </div>
              </div>
            </div>
          </Container>
        </div>
      </header>
      {isHomePage && (
        <div
          className="flex-none"
          style={{ height: 'var(--content-offset)' }}
        />
      )}
    </>
  )
}
